@@ -359,7 +359,7 @@ where
 {
     fn parse(&mut self) -> PResult<ViewportRule> {
         let span = self.input.cur_span()?;
-        let block = self.parse_simple_block()?;
+        let block = self.parse()?;
 
         return Ok(ViewportRule {
             span: span!(self, span.lo),
@@ -432,7 +432,7 @@ where
 {
     fn parse(&mut self) -> PResult<FontFaceRule> {
         let span = self.input.cur_span()?;
-        let block = self.parse_simple_block()?;
+        let block = self.parse()?;
 
         return Ok(FontFaceRule {
             span: span!(self, span.lo),
@@ -533,9 +533,7 @@ where
                 .map(KeyframeBlockRule::AtRule);
         }
 
-        self.parse_simple_block()
-            .map(Box::new)
-            .map(KeyframeBlockRule::Block)
+        self.parse().map(Box::new).map(KeyframeBlockRule::Block)
     }
 }
 
@@ -617,7 +615,7 @@ where
                     query,
                 })
             } else {
-                let declaration = self.parse_declaration()?;
+                let declaration = self.parse()?;
 
                 SupportQuery::Declaration(declaration)
             };
@@ -898,7 +896,7 @@ where
             Token::AtKeyword { .. } => Ok(PageRuleBlockItem::Nested(self.parse()?)),
             _ => {
                 let p = self
-                    .parse_declaration()
+                    .parse()
                     .map(Box::new)
                     .map(PageRuleBlockItem::Declaration)?;
                 eat!(self, ";");
